# Geodashboard

## About
If you are new to this project, you may want to start reading at the Architecture section below for information on how we use branches, how the application is re-used for other projects, as well as application architecture and background.

## Setup

To get up and running quickly, we'll setup your development environment so that your local copy of geodashboard will use the Seagrant Development API.

### Requirements

* Node & NPM
* Grunt
* An account on [Stash](https://opensource.ncsa.illinois.edu/stash/projects/GEOD/repos/geodashboard/browse) where you will push your code. Stash is made by Atlassian and is comparable to GitHub. You can checkout the project without an account, but you will need an account before you can push changes.

#### Node
Ensure that you have a copy of node and npm by running `which node` and `which npm`. If you are not shown a path to these applications, follow the instructions at http://nodejs.org/download/.

#### Grunt
Ensure you have a copy of Grunt installed by running `which grunt`. If you are not shown a path to grunt, run `npm install -g grunt-cli` and refer to the website with any issues at http://gruntjs.com.

#### Installation

	git clone ssh://git@opensource.ncsa.illinois.edu:7999/geod/geodashboard.git
	cd geodashboard
	./sbt run
	
Running `./sbt run` will start the Play framework and will start a web server at [http://localhost:9000](http://localhost:9000)

You will need to run grunt before you can view the project in the browser. Open a second terminal in the geodashboard directory and run the following:


	cd geodashboard
	cd src
	npm install
	grunt dev
	
Running `grunt dev` will configure the application to use http://seagrant-dev.ncsa.illinois.edu for the Clowder API.


## Architecture

You can read more about this project on [NCSA's public wiki](https://opensource.ncsa.illinois.edu/confluence/display/GEOD/The+Geo+Dashboard+Project).

As geodashboard was being built, it was designed for re-use with other projects. Some examples of the projects' implementations are:

* [Great Lakes Monitoring](http://greatlakesmonitoring.org) - http://greatlakesmonitoring.org
* [IMLCZO](http://imlczo.ncsa.illinois.edu/geodashboard) - http://imlczo.ncsa.illinois.edu/geodashboard

Geodashboard consists of several components:


	1. an API for storing and retrieving data called "Clowder"
	2. a GUI for uploading, listing, and modifying data on the Clowder API
	3. a JavaScript front-end for displaying the data visually on a map (ie: geo-dashboard)

This repo contains the code for only the third component, the geodashboard. Development of the geodashboard is typically done running a local version of the Clowder API, though it is possible to point your dashboard to one of the development versions of the Clowder API when getting started.

The entire JavaScript application gets deployed using the [Play Framework](http://playframework.com). What this means is that your html, javascript, and css files will run through a build process that creates a jar file that is deployed to the server. While developing, you will run the play server to host the application, while running grunt to concatenate and rebuild your javascript files when they are changed.

## Project Configuration

In order to re-use the code for multiple projects, there are two important points to be familiar with:

	1. Git Branches
	2. The app.config object

### Git Branches
The project uses the `master` branch as the base branch. Individual projects use named branches when they are customized. For example, the [Sea Grant](http://greatlakesmonitoring.org) project uses the `SEAGRANT` branch, and the [IMLCZO](http://imlczo.ncsa.illinois.edu/geodashboard) project uses the `IMLCZO` branch.

Project branches are never to be merged into master.

Changes to the code base that should apply to all projects should be made to master, and changes that apply only to a project should be merged into the project branch. Changes made to master are regularly merged into the project branches, but project branches are never merged into master.

### The app.config object

The application uses a global namespace called `app`. The app object is a JavaScript object that stores just about everything for the project, and is overridden on project branches to make customizations. Rather than making changes to names and text all throughout the project's files, we create an `app.config` object much like is used with internationalization. The `app.config` object is modified in `/src/javascript/config.js` for each project.

Here's an example of some of the things you'll find in `/src/javascript/config.js`:

	app.config = {
		title : "Sea Grant",
		clowderUrl: "http://seagrant-dev.ncsa.illinois.edu",
		...
	}

And here's an example of how it gets implemented, taken from `/src/javascript/views/AppView.js`:

	# // on route:home, set currentView to the HomeView
    this.router.on("route:home", (function(self) {
		...
        document.title = app.config.title;
        ...   

We're setting the title of the homepage. Instead of using a string like "Sea Grant", we can reference `app.config.title`. This means that the `AppView.js` file (and most other files) are the same for all projects, and most of the changes that have to be made are in the `config.js` file.

## Tests
You will need to run `npm install` from the `/src/` directory to install the dependencies for the test suites.

We are experimenting with tests, but have started using mocha, chai, phantomJS and CasperJS. All tests can be found in `/src/javascript/test`. The `spec` directory contains tests that essentially test the Backbone code that we write; the `casperjs` directory contains tests that will open a browser and proceed to click through a scenario. 

You can run the "spec" tests by running `grunt test` or by simply opening the `/src/javascript/test/test.html` file in the browser. You can run the "casperjs" tests by running `grunt casperjs`.

When writing "spec" tests, follow the examples in `/src/javascript/test/spec/` and source any new scripts in `/src/javascript/test/test.html`. The "casperjs" grunt task will run all files in the "casperjs" directory.